/* Copyright 2004 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: testlevel.as,v 1.7 2004/08/05 01:21:31 tonic Exp $
 * $Revision: 1.7 $
 */

/* Note: playSound function is undocumented as it is currently
 *       only temporarily as it is.
 */


const int ELEVATOR_SCORE = 1000;

int screenWidth, screenHeight;
uint frame = 0;

int score = 0;

GameApp *app;
UIFont *medFont, *smallFont;
uint medFontHeight, smallFontHeight;
PhysicsWorld *pw;

PhysicsObject *objHead, *objNeck, *objTrunk, *objRthigh, *objLthigh, *objRcalf, *objLcalf, *objRfore, *objLfore, *objLarm, *objRarm, *objRfoot, *objLfoot;
float headPosX, headPosY, headPosZ; 

PhysicsObject *objElevator;
int elevAct = 0;
float elevPosX, elevPosY, elevPosZ;
float elevColR, elevColG, elevColB;
float getal = 0;
float initialElevPosX, initialElevPosY, initialElevPosZ; 


bool finished = false;
bool aborted = false;


// Called once on level initialization.
void onInit()
{
    app = getGameApp();
    app->setClearColor(0, 0.7, 1.0);

    screenWidth = app->getScreenWidth();
    screenHeight = app->getScreenHeight();
    medFont = app->getFont("medium");
    smallFont = app->getFont("small");
    medFontHeight = medFont->getFontHeight();
    smallFontHeight = smallFont->getFontHeight();

    pw = app->getPhysicsWorld();
    pw->setGravity(0, 0, -0.5);

    // Ragdoll head
    objHead = pw->findObject("Head");
    objNeck = pw->findObject("Neck");
    objTrunk = pw->findObject("Trunk");
    objRthigh = pw->findObject("RightThigh");
    objLthigh = pw->findObject("LeftThigh");
    objRarm = pw->findObject("RightArm");
    objLarm = pw->findObject("LeftArm");
    objRfore = pw->findObject("RightForearm");
    objLfore = pw->findObject("LeftForearm");
    objRcalf = pw->findObject("RightCalf");
    objLcalf = pw->findObject("LeftCalf");
    objRfoot = pw->findObject("RightFoot");
    objLfoot = pw->findObject("LeftFoot");

    revertColors();
    

    objElevator = pw->findObject("Elevator");
    objElevator->getInitialPosition(elevPosX, elevPosY, elevPosZ);
    objElevator->getInitialPosition(initialElevPosX, initialElevPosY, initialElevPosZ); 		
    objElevator->getInitialColor(elevColR, elevColG, elevColB);


}

void revertColors()
{

    objHead->setColor(1.00000, 0.562500, 0.385417);
    objNeck->setColor(1.00000, 0.562500, 0.385417);
    objTrunk->setColor(0.250000, 0.562500, 1.00000);
    objRthigh->setColor(0.00000, 1.000000, 0.00000);
    objLthigh->setColor(0.00000, 1.000000, 0.00000);
    objRarm->setColor(0.250000, 0.562500, 1.00000);
    objLarm->setColor(0.250000, 0.562500, 1.00000);
    objRfore->setColor(1.00000, 0.562500, 0.385417);
    objLfore->setColor(1.00000, 0.562500, 0.385417);
    objRcalf->setColor(1.00000, 0.562500, 0.385417);
    objLcalf->setColor(1.00000, 0.562500, 0.385417);
    objRfoot->setColor(1.00000, 1.00000, 0.270833);
    objLfoot->setColor(1.00000, 1.00000, 0.270833);
		
}


// Called once for each game logic update before the physics update.
// Parameter 'time' is the simulation time in milliseconds.
void onUpdate(uint time)
{
    ++frame;
    

    float x, y, z = 0;
    objHead->getPosition(x, y, z);
    if (!aborted && z < -10)
    {
        aborted = true;
        setGameState(GAMESTATE_ABORTED);
    }

    
    if (elevAct != 0)
    {
        if (elevAct > 0)
        {
            elevPosZ += getal + 0.13f;
            --elevAct;
	    if ((elevAct == 0) && (elevPosZ < 500))
	    {
		getal += 0.10f;
		elevAct = 100;
	    }
	    else if (elevPosZ >= 500)
            {  
 		elevAct = -100;
		getal = 0;		
	    }
        }
        else
        {
            elevPosZ -= getal + 0.13f;
            ++elevAct;
	    if ((elevAct == 0) && (elevPosZ > initialElevPosZ))
	    {
		getal += 0.10f;
		elevAct = -100;
	    }
	    else if (elevPosZ <= initialElevPosZ)
	    {
		elevAct = 0;
		getal = 0;
	    }	
	}
        
	        
	objHead->getPosition(headPosX, headPosY, headPosZ); 
	objElevator->setPosition(headPosX, headPosY, elevPosZ);
        objElevator->setColor(elevColR, elevColG, elevColB * 1.5f);
    }
    else
    {
	objElevator->getPosition(elevPosX, elevPosY, elevPosZ);
	objElevator->setPosition(headPosX, headPosY, elevPosZ);
    }

    revertColors();

}


// Called when there is a collision between the two given objects.
// Returns true to create contacts or false to ignore collision.
bool onCollision(PhysicsObject *o1, PhysicsObject *o2)
{

    if (o1->getType() == PHYSOBJ_RAGDOLL_BODY) o1->setColor(1,0,0);
    if (o2->getType() == PHYSOBJ_RAGDOLL_BODY) o2->setColor(1,0,0);

    // Check if some object hits the elevator trigger (no collisions).
    if ((o1 == objElevator || o2 == objElevator) && elevAct == 0)// && elevTimer == 0)
    {
        // Move elevator after ragdoll hits the elevator trigger.
        if (o1->getType() == PHYSOBJ_RAGDOLL_BODY ||
            o2->getType() == PHYSOBJ_RAGDOLL_BODY)
        {
            elevAct = 100;
            score += ELEVATOR_SCORE;
            playSound(5); // elevator sound
        }
        // no collision
        return false;
    }


    // generate collision
    return true;
}


// Called when the game view is rendered.
// Might happen more or less frequently than update calls.
void onRender()
{
    const float gap = 70;

    medFont->drawString(float(screenWidth / 2), gap,
    	                   UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM,
                           bstrFormat(elevAct));

    objHead->getPosition(headPosX, headPosY, headPosZ);	
    medFont->drawString(float(screenWidth / 2), 1.5f * gap,
    	                   UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM,
                           "Hoogte poppetje: " + bstrFormat(headPosZ));	

    medFont->drawString(float(screenWidth / 2), 2*gap,
    	                   UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM,
                           "Lifthoogte: " + bstrFormat(elevPosZ));

    
    // draw score
    const float x = float(screenWidth) - gap;
    medFont->drawString(x, gap, UIFONT_ALIGN_RIGHT, bstrFormat(score));
    smallFont->drawString(x, gap + float(medFontHeight),
                          UIFONT_ALIGN_RIGHT, "Score");


    if (aborted)
        drawFailure();
    if (finished)
        drawVictory();
}



void drawFailure()
{
    medFont->drawString(float(screenWidth / 2), float(screenHeight / 2),
                        UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,
                        "You Failed!");
}


void drawVictory()
{
    medFont->drawString(float(screenWidth / 2), float(screenHeight / 2),
                        UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,
                        "Victory!");
    
    medFont->drawString(float(screenWidth / 2),
                        float(screenHeight / 2 + int(medFontHeight) * 5),
                        UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,
                        "Total score: " + bstrFormat(score));
}
